@@ -157,20 +157,10 @@ class Tree(Generic[GenericNode]):
 
     def _ensure_present(
         self,
-        nid: Optional[NodeId],
-        defaults_to_root: bool = False,
-        allow_empty: bool = False,
-    ) -> Optional[NodeId]:
-        # TODO - split method based on intent
-        if nid is None:
-            if not self.is_empty() and defaults_to_root:
-                return self.root
-            if allow_empty:
-                return None
-            raise ValueError("'nid' set to None not supported.")
-        if nid not in self:
+        nid: NodeId,
+    ) -> None:
+        if nid is None or nid not in self:
             raise NotFoundNodeError("Node id <%s> doesn't exist in tree" % nid)
-        return nid
 
     def _validate_node_insertion(self, node: GenericNode) -> None:
         if node.identifier in self._nodes_map.keys():
@@ -183,7 +173,7 @@ class Tree(Generic[GenericNode]):
             # todo validate key
             self._validate_node_insertion(node)
 
-    def _clone_init(self: GenTree, deep: bool) -> GenTree:
+    def _clone_init(self: GenTree, deep: bool, with_nodes: bool) -> GenTree:
         """Method intended to be overloaded, to avoid rewriting whole methods relying on `clone` method when
         inheriting from Tree, so that the way a tree is duplicated is explicit.
 
@@ -193,7 +183,7 @@ class Tree(Generic[GenericNode]):
         >>>         self.tree_description = tree_description
         >>>         self.large_data = large_data
         >>>
-        >>>     def _clone_init(self, deep=False):
+        >>>     def _clone_init(self, deep, with_nodes):
         >>>         return TreeWithComposition(
         >>>             tree_description=self.tree_description,
         >>>             large_data=copy.deepcopy(self.large_data) if deep else self.large_data
@@ -215,7 +205,7 @@ class Tree(Generic[GenericNode]):
         new_root: Optional[NodeId] = None,
     ) -> GenTree:
         """Clone current instance, with or without nodes."""
-        new_tree = self._clone_init(deep)
+        new_tree = self._clone_init(deep=deep, with_nodes=with_nodes)
         if not with_nodes:
             return new_tree
 
@@ -610,9 +600,14 @@ class Tree(Generic[GenericNode]):
         :param reverse: the ``reverse`` param for sorting :class:`Node` objects in the same level
         :return: node ids that satisfy the conditions if ``id_only`` is True, else nodes.
         """
+        if self.is_empty():
+            return None
         if mode not in ("depth", "width"):
             raise NotImplementedError("Traversal mode '%s' is not supported" % mode)
-        nid = self._ensure_present(nid, defaults_to_root=True, allow_empty=True)
+        if nid is None:
+            nid = self.root
+        else:
+            self._ensure_present(nid)
         if nid is not None:
             key, node = self.get(nid)
             filter_pass_node = filter_ is None or filter_(key, node)
@@ -716,30 +711,38 @@ class Tree(Generic[GenericNode]):
         :param is_last_list: list of booleans, each indicating if node is the last yielded one at this depth
         :return: tuple of booleans, node
         """
+        if self.is_empty():
+            return None
+
         is_last_list = is_last_list or []
 
-        nid = self._ensure_present(nid, defaults_to_root=True, allow_empty=True)
-        if nid is not None:
-            key, node = self.get(nid)
-            if filter_ is None or filter_(node):
-                yield tuple(is_last_list), key, node
-                children = [
-                    (child_key, child_node)
-                    for child_key, child_node in self.children(nid)
-                    if filter_ is None or filter_(child_node)
-                ]
-                idxlast: int = len(children) - 1
-                children.sort(key=itemgetter(0), reverse=reverse)
-                for idx, (child_k, child) in enumerate(children):
-                    is_last_list.append(idx == idxlast)
-                    for item in self._iter_nodes_with_location(
-                        nid=child.identifier,
-                        filter_=filter_,
-                        reverse=reverse,
-                        is_last_list=is_last_list,
-                    ):
-                        yield item
-                    is_last_list.pop()
+        nid_: NodeId
+        if nid is None:
+            nid_ = self.root  # type: ignore
+        else:
+            self._ensure_present(nid)
+            nid_ = nid
+
+        key, node = self.get(nid_)
+        if filter_ is None or filter_(node):
+            yield tuple(is_last_list), key, node
+            children = [
+                (child_key, child_node)
+                for child_key, child_node in self.children(nid_)
+                if filter_ is None or filter_(child_node)
+            ]
+            idxlast: int = len(children) - 1
+            children.sort(key=itemgetter(0), reverse=reverse)
+            for idx, (child_k, child) in enumerate(children):
+                is_last_list.append(idx == idxlast)
+                for item in self._iter_nodes_with_location(
+                    nid=child.identifier,
+                    filter_=filter_,
+                    reverse=reverse,
+                    is_last_list=is_last_list,
+                ):
+                    yield item
+                is_last_list.pop()
 
     @staticmethod
     def _line_repr(
@@ -816,7 +819,10 @@ class Tree(Generic[GenericNode]):
         if self.is_empty():
             return self.insert(new_tree, parent_id=None)
 
-        nid = self._ensure_present(nid, defaults_to_root=True)
+        if nid is None:
+            nid = self.root
+        else:
+            self._ensure_present(nid)
 
         if new_tree.root is None:
             # not possible, only for typing
